# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

from . import ATCamera
from . import ATDome
from . import ATHexapod
from . import ATMCS
from . import ATMonochromator
from . import ATPneumatics
from . import ATPtg
from . import ATSpectrograph
from . import ATThermoelectricCooler
from . import Electrometer
from . import FiberSpectrograph
from . import Guider
from . import LinearStage
from . import MTAOS
from . import MTDome
from . import MTHexapod
from . import MTM1M3
from . import MTM2
from . import MTMount
from . import MTPtg
from . import MTRotator
from . import PMD
from . import Script
from . import ScriptQueue
from . import TunableLaser
from . import Watcher



# This file is part of ts_idl.
#
# Developed for the LSST Telescope and Site Systems.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import unittest

from lsst.ts import idl


class UtilsTestCase(unittest.TestCase):
    def test_get_pkg_root(self):
        pkg_root = idl.get_pkg_root()
        self.assertEqual(pkg_root.name, "ts_idl")

    def test_get_idl_dir(self):
        pkg_root = idl.get_idl_dir()
        self.assertEqual(pkg_root.name, "idl")

    def test_enums_modules(self):
        enum_module_names = set(
            name for name in dir(idl.enums) if not name.startswith("_")
        )
        # These are the names as of 2021-02-22
        # Treat it as a subset to avoiding having to update this test
        # every time we add a module.
        expected_names_subset = set(
            (
                "ATCamera",
                "ATDome",
                "ATHexapod",
                "ATMCS",
                "ATMonochromator",
                "ATPneumatics",
                "ATPtg",
                "ATSpectrograph",
                "ATThermoelectricCooler",
                "Electrometer",
                "FiberSpectrograph",
                "Guider",
                "LinearStage",
                "MTAOS",
                "MTDome",
                "MTHexapod",
                "MTM1M3",
                "MTM2",
                "MTMount",
                "MTPtg",
                "MTRotator",
                "PMD",
                "Script",
                "ScriptQueue",
                "TunableLaser",
                "Watcher",
            )
        )
        self.assertTrue(expected_names_subset <= enum_module_names)


if __name__ == "__main__":
    unittest.main()



# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation either version 3 of the License or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

__all__ = [
    "AxisMotionState",
    "Commander",
    "DeployableMotionState",
    "ElevationLockingPinMotionState",
    "LimitsMask",
    "PowerState",
    "System",
]

import enum


class AxisMotionState(enum.IntEnum):
    """Motion state of azimuth elevation and camera cable wrap."""

    STOPPING = 0
    STOPPED = 1
    MOVING_POINT_TO_POINT = 2
    JOGGING = 3
    TRACKING = 4
    TRACKING_PAUSED = 5


class Commander(enum.IntEnum):
    """Who commands the low-level controller."""

    NONE = 0
    CSC = 1
    EUI = 2
    HDD = 3


class DeployableMotionState(enum.IntEnum):
    """Motion state of deployable systems.

    These include the deployable platform, mirror covers,
    and mirror cover locks.
    """

    RETRACTED = 0
    DEPLOYED = 1
    RETRACTING = 2
    DEPLOYING = 3
    LOST = 4


class ElevationLockingPinMotionState(enum.IntEnum):
    """Position of elevation locking pin."""

    LOCKED = 0
    TEST = 1
    UNLOCKED = 2
    MOVING = 3
    MISMATCH = 4


class LimitsMask(enum.IntFlag):
    """Bit masks for the various limits.

    * L1 = software limit
    * L2 = direction inhibit
    * L3 = cut power
    """

    L1_MIN = 0x01
    L1_MAX = 0x02
    L2_MIN = 0x03
    L2_MAX = 0x04
    L3_MIN = 0x05
    L3_MAX = 0x06
    ADJUSTABLE_L1_MIN = 0x07
    ADJUSTABLE_L1_MAX = 0x08
    OPERATIONAL_L1_MIN = 0x09
    OPERATIONAL_L1_MAX = 0x10


class PowerState(enum.IntEnum):
    """Power state of a system or motion controller.

    Also used for motion controller state.

    Note that only a few systems (and no motion controllers)
    use TURNING_ON and TURNING_OFF. The oil supply system is one.
    """

    OFF = 0
    ON = 1
    FAULT = 2
    TURNING_ON = 3
    TURNING_OFF = 4
    UNKNOWN = 15


class System(enum.IntEnum):
    AZIMUTH = 0
    ELEVATION = 1
    CAMERA_CABLE_WRAP = 2
    BALANCE = 3
    MIRROR_COVERS = 4
    MIRROR_COVER_LOCKS = 5
    AZIMUTH_CABLE_WRAP = 6
    LOCKING_PINS = 7
    DEPLOYABLE_PLATFORMS = 8
    OIL_SUPPLY_SYSTEM = 9
    AZIMUTH_DRIVES_THERMAL = 10
    ELEVATION_DRIVES_THERMAL = 11
    AZ0101_CABINET_THERMAL = 12
    MODBUS_TEMPERATURE_CONTROLLERS = 13
    MAIN_CABINET = 14
    MAIN_AXES_POWER_SUPPLY = 15
    TOP_END_CHILLER = 16



# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

__all__ = ["FilterPosition", "DisperserPosition", "Status", "Error"]

import enum


class FilterPosition(enum.IntEnum):
    FILTER0 = 1
    FILTER1 = 2
    FILTER2 = 3
    FILTER3 = 4
    INBETWEEN = 5


class DisperserPosition(enum.IntEnum):
    DISPERSER0 = 1
    DISPERSER1 = 2
    DISPERSER2 = 3
    DISPERSER3 = 4
    INBETWEEN = 5


class Status(enum.IntEnum):
    HOMING = 1
    MOVING = 2
    STATIONARY = 3
    NOTINPOSITION = 4


class Error(enum.IntEnum):
    NONE = 1
    BUSY = 2
    NOTINITIALIZED = 3
    MOVETIMEOUT = 4



# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

__all__ = ["Grating", "Slit", "LightStatus", "Device"]

import enum


class Grating(enum.IntEnum):
    BLUE = 1
    RED = 2
    MIRROR = 3


class Slit(enum.IntEnum):
    ENTRY = 1
    EXIT = 2


class LightStatus(enum.IntEnum):
    ON = 1
    OFF = 2


class Device(enum.IntEnum):
    MONOCHROMATOR = 1
    LIGHTSOURCE = 2
    THERMOELECTRICCOOLER = 3


class DetailedState(enum.IntEnum):
    NOT_ENABLED = 1
    READY = 2
    CHANGING_WAVELENGTH = 3
    CALIBRATING_WAVELENGTH = 4
    POWERING = 5
    SELECTING_GRATING = 6
    CHANGING_SLIT_WIDTH = 7
    UPDATING_SETUP = 8


class Status(enum.IntEnum):
    SETTING_UP = 1
    READY = 2
    OFFLINE = 3
    FAULT = 4



import enum


class Subsystem(enum.Enum):
    GISCPUINPUTS = enum.auto
    GISCPUOUTPUTS = enum.auto
    GISCPURESERVE = enum.auto
    ACCESSFIREEARTHQUAKEINPUTS = enum.auto
    ACCESSFIREEARTHQUAKEOUTPUTS = enum.auto
    ACCESSFIREEARTHQUAKEFREE = enum.auto
    LASERINPUTS = enum.auto
    LASEROUTPUTS = enum.auto
    LASERFREE = enum.auto
    M2INPUTS = enum.auto
    M2OUTPUTS = enum.auto
    M2FREE = enum.auto
    PFLOWINPUTS = enum.auto
    PFLOWOUTPUTS = enum.auto
    PFLOWFREE = enum.auto
    AUXCPUINPUTS = enum.auto
    AUXCPUOUTPUTS = enum.auto
    DOMEINPUTS = enum.auto
    DOMEOUTPUTS = enum.auto
    M1M3CPUINPUTS = enum.auto
    M1M3CPUOUTPUTS = enum.auto
    TMACPUINPUTS = enum.auto
    TMACPUOUTPUTS = enum.auto
    SAFETYCAUSESONE = enum.auto
    SAFETYCAUSESTWO = enum.auto
    SAFETYCAUSESONEOVERRIDE = enum.auto
    SAFETYCAUSESTWOOVERRIDE = enum.auto
    SAFETYEFFECTSONE = enum.auto
    SAFETYEFFECTSTWO = enum.auto



# This file is part of ts_idl.
#
# Developed for the LSST Telescope and Site Systems.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__all__ = ["Location", "ScriptProcessState"]

import enum


class Location(enum.IntEnum):
    """Location constants for adding and moving scripts."""

    FIRST = 1
    LAST = 2
    BEFORE = 3
    AFTER = 4


class ScriptProcessState(enum.IntEnum):
    """ScriptQueue script.processState event constants."""

    UNKNOWN = 0
    LOADING = 1
    CONFIGURED = 2
    RUNNING = 3
    DONE = 4
    LOADFAILED = 5
    CONFIGUREFAILED = 6
    TERMINATED = 7
